package edu.uiuc.ncsa.myproxy.oa4mp; /**
 * Created with IntelliJ IDEA.
 * User: wedwards
 * Date: 11/14/12
 * Time: 3:11 PM
 * This class allows us to call some methods from the BouncyCastle libraries to test whether an exception occurs
 * To use this class, it is necessary to have globus certificates set up in your home directory
 * (~/.globus/certificates)
 */

import edu.uiuc.ncsa.myproxy.MyProxyServiceFacade;
import edu.uiuc.ncsa.myproxy.ServiceFacadeConfiguration;
import edu.uiuc.ncsa.security.util.pkcs.CertUtil;
import edu.uiuc.ncsa.security.util.pkcs.KeyUtil;
import edu.uiuc.ncsa.security.util.ssl.SSLKeystoreConfiguration;
import junit.framework.TestCase;
import org.bouncycastle.jce.PKCS10CertificationRequest;
import org.globus.gsi.X509ExtensionSet;
import org.globus.gsi.bc.BouncyCastleCertProcessingFactory;
import org.globus.gsi.proxy.ext.ProxyCertInfo;
import org.globus.gsi.proxy.ext.ProxyCertInfoExtension;
import org.globus.gsi.proxy.ext.ProxyPolicy;

import java.security.KeyPair;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.LinkedList;

import static org.globus.gsi.GSIConstants.CertificateType.GSI_4_LIMITED_PROXY;


public class BouncyCastleTest extends TestCase {

    public static void testBouncyCastle() {
        PKCS10CertificationRequest certReq;
        // drum roll. This is messy without these:
        KeyPair keyPair;
        try {
            keyPair = KeyUtil.generateKeyPair();
            certReq = CertUtil.createCertRequest(keyPair);

            // configure a MyProxy facade to talk to Myproxy. This example points to the one on ncsa.
            ServiceFacadeConfiguration sfc = new ServiceFacadeConfiguration("myproxy.ncsa.uiuc.edu", 7512, new HashMap<String, Integer>());
            SSLKeystoreConfiguration keystoreConfiguration = new SSLKeystoreConfiguration();
            keystoreConfiguration.setUseDefaultJavaKeyStore(true);
            MyProxyServiceFacade mp = new MyProxyServiceFacade(sfc, keystoreConfiguration);
            // This is so we have a place to put the cert chain.
            LinkedList<X509Certificate> certs = new LinkedList<X509Certificate>();

            // When running the class, specify these values as parameters, e.g.
            // "-ea -Dmyproxy:username=yourusername -Dmyproxy:password=yourpassword"
            // The "0L" is important -- if you just pass in "0", it'll compile, but you'll get a strange network error
            certs.addAll(mp.getCerts(System.getProperty("myproxy:username"), System.getProperty("myproxy:password"), 0L, certReq.getDEREncoded()));

            // the setup to use jGlobus. These are the constants you
            // need to set up the limited proxy factory.
            Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
            ProxyPolicy policy = new ProxyPolicy(ProxyPolicy.LIMITED);
            ProxyCertInfo proxyCertInfo = new ProxyCertInfo(policy);
            X509ExtensionSet extSet = new X509ExtensionSet();
            extSet.add(new ProxyCertInfoExtension(proxyCertInfo));
            BouncyCastleCertProcessingFactory factory =
                    BouncyCastleCertProcessingFactory.getDefault();

            KeyPair keyPair2 = KeyUtil.generateKeyPair();
            // The actual call:

            certs.addFirst(factory.createProxyCertificate(certs.getLast(),
                    keyPair2.getPrivate(), certReq.getPublicKey(),
                    (int) 86400, // get one for a day
                    GSI_4_LIMITED_PROXY, extSet, null));

        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    // Just for testing
    public static void main(String[] args) {
              BouncyCastleTest.testBouncyCastle();
              System.out.println("It ran");
    }

}
