package edu.uiuc.ncsa.myproxy.oa4mp;

import edu.uiuc.ncsa.myproxy.MyProxyServiceFacade;
import edu.uiuc.ncsa.myproxy.ServiceFacadeConfiguration;
import edu.uiuc.ncsa.security.util.TestBase;
import edu.uiuc.ncsa.security.util.pkcs.CertUtil;
import edu.uiuc.ncsa.security.util.pkcs.KeyUtil;
import edu.uiuc.ncsa.security.util.ssl.SSLKeystoreConfiguration;
import org.bouncycastle.jce.PKCS10CertificationRequest;
import org.globus.gsi.X509ExtensionSet;
import org.globus.gsi.bc.BouncyCastleCertProcessingFactory;
import org.globus.gsi.proxy.ext.ProxyCertInfo;
import org.globus.gsi.proxy.ext.ProxyCertInfoExtension;
import org.globus.gsi.proxy.ext.ProxyPolicy;
import org.junit.Test;

import java.security.KeyPair;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.LinkedList;

import static org.globus.gsi.GSIConstants.CertificateType.GSI_4_LIMITED_PROXY;


/**
 * <p>Created by Jeff Gaynor<br>
 * on 11/13/12 at  9:19 AM
 */
public class ProxyLimitedTest extends TestBase {
    @Test

    public void testProxyLimited() throws Exception {
        // drum roll. This is messy without these:
        PKCS10CertificationRequest certReq;
        KeyPair keyPair;
        keyPair = KeyUtil.generateKeyPair();
        certReq = CertUtil.createCertRequest(keyPair);

        // configure a MyProxy facade to talk to Myproxy. This example points to the one on go.teragrid.org.
        ServiceFacadeConfiguration sfc = new ServiceFacadeConfiguration("myproxy.teragrid.org", 7514, new HashMap<String, Integer>());
        SSLKeystoreConfiguration keystoreConfiguration = new SSLKeystoreConfiguration();
        keystoreConfiguration.setUseDefaultJavaKeyStore(true);
        MyProxyServiceFacade mp = new MyProxyServiceFacade(sfc, keystoreConfiguration);

        // This is so we have a place to put the cert chain.
        LinkedList<X509Certificate> certs = new LinkedList<X509Certificate>();
        // Note the next call requires that the 3 rd parameter -- the cert lifetime 00 be a long or this will
        // be mis-interpreted as an int (port number) giving you an exception.

        certs.addAll(mp.getCerts(System.getProperty("myproxy:username"), System.getProperty("myproxy:password"), certReq.getDEREncoded(), 0));

        assert !certs.isEmpty() : "Error: MyProxy did not return a usable cert.";


        // now for the ugly setup to use jGlobus. These are all the nasty constants you
        // need to set up the limited proxy factory. You should just believe work and copy them:
        Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
        ProxyPolicy policy = new ProxyPolicy(ProxyPolicy.LIMITED);
        ProxyCertInfo proxyCertInfo = new ProxyCertInfo(policy);
        X509ExtensionSet extSet = new X509ExtensionSet();
        extSet.add(new ProxyCertInfoExtension(proxyCertInfo));
        BouncyCastleCertProcessingFactory factory =BouncyCastleCertProcessingFactory.getDefault();
        KeyPair keyPair2 = KeyUtil.generateKeyPair();

        int certsLength = certs.size();
        // The actual call:
        certs.addFirst(factory.createProxyCertificate(certs.getLast(),
                keyPair2.getPrivate(), certReq.getPublicKey(),
                (int) 86400, // get one for a day
                GSI_4_LIMITED_PROXY, extSet, null));

        assert certs.size() == certsLength+1 : "Error: limited proxy certificate not created.";
    }
}
