package edu.uiuc.ncsa.myproxy.oa4mp;


import edu.uiuc.ncsa.myproxy.oa4mp.server.DSTransaction;
import edu.uiuc.ncsa.security.core.util.BasicIdentifier;
import edu.uiuc.ncsa.security.delegation.storage.Client;
import edu.uiuc.ncsa.security.delegation.storage.TransactionStore;
import edu.uiuc.ncsa.security.delegation.token.AccessToken;
import edu.uiuc.ncsa.security.delegation.token.AuthorizationGrant;
import edu.uiuc.ncsa.security.delegation.token.Verifier;
import org.junit.Test;

import java.net.URI;


/**
 * <p>Created by Jeff Gaynor<br>
 * on May 19, 2011 at  12:03:54 PM
 */
public abstract class TransactionStoreTest extends StoreTest{

    @Override
    public void checkStoreClass() throws Exception {
         testClassAsignability(getStore());
    }

    public TransactionStore getStore() throws Exception {
        return getTSProvider().getTransactionStore();
    }

    protected AuthorizationGrant newAG(String... x) {
        AuthorizationGrant ag = getTSProvider().getTokenForge().getAuthorizationGrant(x);
        // The forge may return a shared secret. Since we never use this in OA4MP, make sure it is null
        // or you will get false test results since the secret won't be stored.
        ag.setSharedSecret(null);
        return ag;
    }

    protected Verifier newVerifier(String... x) {
        return getTSProvider().getTokenForge().getVerifier(x);
    }

    protected AccessToken newAT(String... x) {
        AccessToken  at = getTSProvider().getTokenForge().getAccessToken(x);
        at.setSharedSecret(null);
        return at;
    }

    @Test
    public void testServiceTransaction() throws Exception {
        DSTransaction dsTransaction = (DSTransaction) getStore().create();
        dsTransaction.setCallback(URI.create("http://callback"));

        dsTransaction.setLifetime(10 * 60 * 60 * 1000); // set lifetime to 10 hours (stored in ms!)
        dsTransaction.setUsername("FakeUserName");

        Client client = getTSProvider().getClientStore().create();
        client.setIdentifier(new BasicIdentifier(URI.create("test:client:1d/" + System.currentTimeMillis())));
        dsTransaction.setAuthorizationGrant(newAG());
        dsTransaction.setAuthGrantValid(false);
        client.setName("service test name #" + System.nanoTime());
        getStore().save(dsTransaction);
        assert getStore().containsKey(dsTransaction.getIdentifier());
        assert dsTransaction.equals(getStore().get(dsTransaction.getIdentifier()));
        assert dsTransaction.equals(getStore().get(dsTransaction.getAuthorizationGrant()));
        // now emulate doing oauth type transactions with it.
        // First leg sets the verifier and user

        String r = getRandomString(12);
        dsTransaction.setVerifier(newVerifier());
        getStore().save(dsTransaction);

        assert dsTransaction.equals(getStore().get(dsTransaction.getVerifier()));
        // next leg creates the access tokens and invalidates the temp credentials
        dsTransaction.setAccessToken(newAT());
        dsTransaction.setAuthGrantValid(false);
        dsTransaction.setAccessTokenValid(true);
        getStore().save(dsTransaction);
        assert dsTransaction.equals(getStore().get(dsTransaction.getIdentifier()));
        assert dsTransaction.equals(getStore().get(dsTransaction.getAccessToken()));
        dsTransaction.setAccessTokenValid(false);
        getStore().save(dsTransaction);
        assert dsTransaction.equals(getStore().get(dsTransaction.getIdentifier()));
        //and we're done
        getStore().remove(dsTransaction.getIdentifier());
        assert !getStore().containsKey(dsTransaction.getIdentifier());
    }
}
